// beddll.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"


#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif



#include "math.h"

extern "C" { 

__declspec( dllexport ) double computeShear(double, double, double, double, int, int, int, int, long int, double, double, double, double, double, double); 
__declspec( dllexport ) double computeCriticalShear(double, double, double, double, int, int, double *, double *, \
													int, int, int, long int, int, double, double, double);
__declspec( dllexport ) double computeBedSlopeCorrection (double, double, double, \
					    double, double, double, double, int, int, int, long int); 
__declspec( dllexport ) double computeBedSlopeCorrection1 (double, double, double, \
					    double, double, int, int, int, long int); 
__declspec( dllexport ) double computeBedConcentration ( double, double, double, \
		int, int, double, double, double, double, double, double, \
		double, double, double, double, double, double, double, \
		double, double, double, int, double, double, double, int, int, int, long int, double);
__declspec( dllexport ) double computeBedConcentration1 ( double, double, double, \
		int, int, double, double, double, double, double, double, \
		double, double, double, double, double, double, double, \
		double, double, double, int, double, double, double, int, int, int, \
		long int, double, double, double *, double *, double, int);
__declspec( dllexport ) double computeBedForms(double, double, double, int, double, double, double, double, double, \
											   double, double, int, int, int, long int, double, double, double);
__declspec( dllexport ) double computeBedRoughness(double, int, double, double,\
						int, double, double, double, int, int, int, long int, double);
}


/**************************************************************************/
/* Do not change anything above these lines. The contents of the six ******/
/* functions given below can be changed. **********************************/
/**************************************************************************/


/***************************************************************/
/****** function 1: computation of the bed shear stress ********/
/****** the function is called before the loop over all ********/
/****** the sediment sizes *************************************/
/***************************************************************/

// variables: 
// kin: turbulent kinetic energy
// rough: bed roughness
// d90: d90 of the bed sediment grain size distribution
// d50: d50 of the bed sediment grain size distribution
// d_mean: mean diameter of the bed grain size distribution
// FlagRoughness: index for choice of different algorithms
// shear: bed shear stress;
// i: index for grid cell in the streamwise direction. Added 11/2-05
// j: index for grid cell in the lateral direction. Added 11/2-05
// CellID: cell number for unstructured grid. zero in a structured grid. Added 11/2-05
// GlobalIter: global iteration number. Added 11/2-05	
// vel_2: velocity in bed cell
// vel_3: velocity in cell above the bed cell
// dist_2: distance from bed to center of bed cell
// dist_3: distance from bed to center of cell above the bed cell
// eddyviscbed: turbulent eddy viscosity in bed cell
	
double computeShear(double kin, double rough, double d90, double d50, int FlagRoughness, 
					int i, int j, int CellID, long int GlobalIter, double d_mean, 
					double vel_2, double vel_3, double dist_2, double dist_3, double eddyviscbed) {

		double shear, ratiomid;

// alternative computation of bed shear, based on wall laws and water velocity
// double uplus;
// uplus = 2.5*ln(30.0*dist_2/rough);
// shear = 1000.0*pow(vel_2/uplus,2.0);

	    shear = 300.0 * kin;
	    if(FlagRoughness == 4 || FlagRoughness == 5) {
			ratiomid = (double)min(1.0,3.0*d90/(rough+1.0e-20));
//			ratiomid = pow(ratiomid, 0.3333333); changed 7/9-00
			ratiomid = pow(ratiomid, 0.25);
			shear *= ratiomid;
			}
		return shear;
	}



/***************************************************************/
/**** a loop is started here, over all the sediment sizes ******/
/**** the following three functions are inside the loop ********/
/***************************************************************/


/***************************************************************/
/**** computation of Shield's coefficient and critical shear ***/
/**** stress for a sediment size *******************************/
/***************************************************************/

// variables: 
// shieldscoeff: Input parameter given on the F 8? data set
// size: sediment grain size, given on the S data set
// kinvisc: kinetic viscosity of water. default: 10e-6
// density: specific desity of sediments, given on the F 11 data set
// shear: bed shear stress, as computed in previous function

// additions 16/9-03: 

// isize: the size that is computed now
// lnumber: number of sediment sizes
// *gsize, pointer to size distribution
// *frac: pointer to array with size fractions of the sediments

// i: index for grid cell in the streamwise direction. Added 11/2-05
// j: index for grid cell in the lateral direction. Added 11/2-05
// CellID: cell number for unstructured grid. zero in a structured grid. Added 11/2-05
// GlobalIter: global iteration number. Added 11/2-05	
// F169Flag: Integer on the F 169 data set: Used for hiding/exposure function	
// F169Param: Float on the F 169 data set: Used for hiding/exposure function
// d50: d50 of the bed sediment grain size distribution
// d_mean: mean diameter of the bed grain size distribution
	

double computeCriticalShear(double shieldscoeff, double kinvisc,
	   double density, double shear, int isize, int lnumber,
	   double *gsize, double *frac, int i, int j, int CellID, long int GlobalIter, int F169Flag, double F169Param,
	   double d50, double d_mean) {

	double alpha, eqshear;

    if(shieldscoeff > 0.0) return gsize[isize] * shieldscoeff * ( density - 1.0 ) * 9810.0;

	alpha = sqrt(shear*0.001) * gsize[isize] / kinvisc;
	if(alpha > 500.0) alpha = 0.0624927;
	else if(alpha < 0.2) alpha = 0.6; // to prevent overflow
	else {
		alpha = log10(alpha);
		alpha = -0.99863612 - 0.92539586 * alpha + 0.54283631 * alpha * alpha
			- 0.084544063 * alpha * alpha * alpha;
		alpha = pow(10.0,alpha);
		}
	eqshear = gsize[isize] * alpha * ( density - 1.0 ) * 9810.0 + 1.0e-20;
 
    return eqshear;

   }




////////////////////////////////////////////////////////////////////
///// correction for sloping bed 
///// only called if F 7 B is used 
////////////////////////////////////////////////////////////////////
// 12. July 2011: if the 2nd parameter on the F 285 data set is 1, then
// the computeBedSlopeCorrection1 function is used. Otherwise, the 
// computeBedSlopeCorrection function is used. 
////////////////////////////////////////////////////////////////////

// variables: 
// vectorX: component in x direction of unit vector normal to the bed surface
// vectorY: component in y direction of unit vector normal to the bed surface
// vectorZ: component in z direction of unit vector normal to the bed surface
// velocityX: velocity in x direction in bed cell
// velocityY: velocity in y direction in bed cell
// param1: parameter in formula
// param2: parameter in formula
// i: index for grid cell in the streamwise direction. Added 11/2-05
// j: index for grid cell in the lateral direction. Added 11/2-05
// CellID: cell number for unstructured grid. zero in a structured grid. Added 11/2-05
// GlobalIter: global iteration number. Added 11/2-05	
// anglecross: bed angle normal to the flow direction, 12/7-11 
// anglelong: bed angle in the flow direction, 12/7-11


double computeBedSlopeCorrection (double vectorX, double vectorY, double vectorZ, 
								  double velocityX, double velocityY, double param1, 
								  double param2, int i, int j, int CellID, long int GlobalIter) {
		double transverseangle, sinalpha, angleofrepose,term1; 

        sinalpha = vectorX * velocityX + vectorY * velocityY;
        sinalpha /= (sqrt(vectorX*vectorX+vectorY*vectorY)
			* sqrt(velocityX*velocityX+velocityY*velocityY) + 1.0e-20);
        angleofrepose = (1.0 + sinalpha) * 0.5;
        angleofrepose = param1 * (1.0-angleofrepose) + angleofrepose * param2; // angleofrepose is 1/tan(angle of repose)
        transverseangle = acos(vectorZ);
        term1 = - sin(transverseangle) * sinalpha * angleofrepose; // 
        term1 += sqrt((double)max(0.01, term1*term1 + pow(cos(transverseangle),2)*(1.0 - pow(tan(transverseangle)*angleofrepose,2.0))));
        term1  = (double)max(term1, 0.2);  
		return term1;
        }

double computeBedSlopeCorrection1 (double anglelong, double anglecross, double vectorZ, 
			  double param1, double param2, int i, int j, int CellID, long int GlobalIter) {

		double transverseangle, angleofrepose,term1; 

        angleofrepose = (1.0 + anglelong) * 0.5;
        angleofrepose = param1 * (1.0-angleofrepose) + angleofrepose * param2; // angleofrepose is 1/tan(angle of repose)
        transverseangle = acos(vectorZ);
//		transverseangle = anglecross;  alternative
        term1 = - sin(transverseangle) * anglelong * angleofrepose; // 
        term1 += sqrt((double)max(0.01, term1*term1 + pow(cos(transverseangle),2)*(1.0 - pow(tan(transverseangle)*angleofrepose,2.0))));
        term1  = (double)max(term1, 0.2);  
		return term1;
        }

////////////////////////////////////////////////////////////////////
/////// function to compute bed concentration //////////////////////
////////////////////////////////////////////////////////////////////
// 14. July 2011: if the 1st parameter on the F 285 data set is 1, then
// the computeBedConcentration1 function is used. Otherwise, the 
// computeBedConcentration function is used. 
////////////////////////////////////////////////////////////////////

// variables: 
// shear: bed shear stress
// eqshear: critical shear stress for particle movement
// size: sediment particle diameter in meters
// TSCbedrouse: integer, the first parameter on the F 60 data set
// BedLoadFlag: integer from the F 84 data set. 0 is suspended load
//              only, 1 is bed load only, 2 is both bed load and 
//              suspended load
// ConcFormCoeff: parameter in van Rijn's suspended load formula
//              default: 0.015, given on the F 6 data set
// ConcFormTstar: parameter in van Rijn's suspended load formula
//              default: 1.5, given on the F 6 data set.
// ConcFormDstar: parameter in van Rijn's suspended load formula
//              default: 0.3, given on the F 6 data set.
// kdist: half the height of the bed cell
// depth: water depth 
// dgraininit: constant in van Rijn's formula
// roughness: bed roughness
// HRzmax: max z value for Hunter-Rouse's formula, given as the first
// parameter on the F 110 data set
// HRmulmin: minimum multiplication factor from the Hunter-Rouse's formula
// for bed concentration extrapolation/interpolation, given as the 
// second parameter on the F 110 data set
// fallvel: fall velocity of the sediment size, given on the S data set
// MaxBedConc: maximum bed concentration, set to 0.1
// ConcFormBCoeff: parameter in van Rijn's formula for bed load, given as 
//           the first parameteter on the F 83 data set. default: 0.053
// ConcFormBTstar: parameter in van Rijn's formula for bed load, given as
//           the second parameter on the F 83 data set. default: 2.1           
// ConcFormBDoeff: parameter in van Rijn's formula for bed load, given as 
//           the third parameteter on the F 83 data set. default: 0.30
// ConcFormBD50: parameter in van Rijn's formula for bed load, given as
//           the fourth parameter on the F 83 data set. default: 1.5           
// rijnbedloadcoeff: constant in van Rijn's formula for bed load. 
//                 equal to sqrt(1.65*9.81)
// FlagRoughness: integer for choice of method to compute the bed roughness
//                given on the F 90 data set
// velocityX: velocity in x direction in bed cell
// velocityY: velocity in y direction in bed cell
// i: index for grid cell in the streamwise direction. Added 11/2-05
// j: index for grid cell in the lateral direction. Added 11/2-05
// CellID: cell number for unstructured grid. zero in a structured grid. Added 11/2-05
// GlobalIter: global iteration number. Added 11/2-05	
// d50: d50 of the bed sediment grain size distribution
// d_mean: mean diameter of the bed grain size distribution
// solidfraction: solid fraction in the top layer
// *gsize, pointer to size distribution in the top layer
// *frac: pointer to array with size fractions of the sediments in the top layer
// conc: concentration in the cell in the previous time step
// lnumber: total number of sediment sizes


double computeBedConcentration ( double shear, double eqshear, double size, 
		int TSCbedrouse, int BedLoadFlag, double ConcFormCoeff, double ConcFormTstar,
		double ConcFormDstar, double kdist, double depth, double dgraininit, 
		double roughness, double HRzmax, double HRmulmin, double fallvel, 
		double MaxBedConc, double ConcFormBCoeff, double ConcFormBTstar, 
		double ConcFormBDstar, double ConcFormBD50, double rijnbedloadcoeff, 
		int FlagRoughness, double velocityX, double velocityY, double d50, int i, int j, 
		int CellID, long int GlobalIter, double d_mean) {

		double tshear, dgrain, konsform, a, bb, a_bed, deltashear, z, del;

		deltashear = shear - eqshear;
	    if ( deltashear < 0.000001 ) return 0.0;

        tshear = deltashear / eqshear;
        dgrain = size * dgraininit;

/*
	if(TSCbedrouse == 1) {
		a_bed = (double)max(3.0*d90,0.5*Bedformheight[b]);
		a_bed = (double)max(a_bed, 0.01 * depth);
	} else a_bed = kdist;

*/		
		if(TSCbedrouse == 1 && BedLoadFlag != 1) a_bed = (double)max(roughness,0.01 * depth); // forandret 30/10-06
		else a_bed = kdist; 

// correction for bed load 

	if(BedLoadFlag == 0 || BedLoadFlag == 2) {
//	        konsform = ConcFormCoeff * size * pow(tshear,ConcFormTstar)
//	                / (a_bed * pow(dgrain,ConcFormDstar));
		if(size < 0.0005) konsform = 8.6e-6; // test 13/3-10
		else konsform = 0.0; 

// corrections from distance to bed, after Hunter-Rouse formula ************/

        if(TSCbedrouse == 1 && BedLoadFlag != 1) {
		  a = (double)min(roughness, 0.95 * depth);
//        a = (double)max(a, 0.01 * d);
          bb = kdist - a;
          if(bb > 1.0e-5)  {
             z = (double)min(HRzmax,fallvel * 2.5 / sqrt(0.001 * shear + 1.0e-20));
             del = ((depth - kdist) / kdist) * (a  / (depth - a));
             del = pow(del,z);
             del = (double)min(del , 1.0);
             del = (double)max(del , HRmulmin);
             konsform *= del;
             }
          else if(bb < -1.0e-5)  {
             z = (double)min(HRzmax,fallvel * 2.5 / sqrt(0.001 * shear + 1.0e-20));
             del = ((depth - a) / a) * (kdist  / (depth - kdist));
             del = pow(del,z);
             del = (double)min(del , 1.0);
             del = (double)max(del , HRmulmin);
             konsform /= del;
             }
          }
     } // end if susp. load

// bed load

	if(BedLoadFlag == 1) konsform = 0.0;
	if(BedLoadFlag == 1 || BedLoadFlag == 2) 
	        konsform += ConcFormBCoeff * pow(size,ConcFormBD50) * pow(tshear,ConcFormBTstar)
			* rijnbedloadcoeff / (2.0 * kdist * pow(dgrain,ConcFormBDstar) *
			sqrt(velocityX*velocityX+velocityY*velocityY) + 1.0e-20);

/***************************************************************/
/****** Gessler's armouring layer theory 9/6-00. ****************/
/****** This is not tested at all. You may decleare *************/
/****** your own variables and compute this effect *************/
/****** according to your chosen formula ***********************/
/***************************************************************/

	if(FlagRoughness == 5) {
/*		x = (eqshear/shear - 1.0) / sigma;
		T = 1.0 / ( 1.0 + 0.2316419 * fabs(x));
		P = T*(0.31938153+T*(-0.35653782+T*(1.781477937
		+T*(-1.821255978+T*1.3130274429))));
		P = (0.3989422804*P*exp(-0.5*x*x));
		konsform *= 1.0-P;
*/		}

    return (double)min(konsform, MaxBedConc);

}

double computeBedConcentration1 ( double shear, double eqshear, double size, 
		int TSCbedrouse, int BedLoadFlag, double ConcFormCoeff, double ConcFormTstar,
		double ConcFormDstar, double kdist, double depth, double dgraininit, 
		double roughness, double HRzmax, double HRmulmin, double fallvel, 
		double MaxBedConc, double ConcFormBCoeff, double ConcFormBTstar, 
		double ConcFormBDstar, double ConcFormBD50, double rijnbedloadcoeff, 
		int FlagRoughness, double velocityX, double velocityY, double d50, int i, int j, 
		int CellID, long int GlobalIter, double d_mean, 
		double solidfraction, double *frac, double *gsize, double conc, int lnumber) {

double konsform = 0.0001;
return konsform;

}


/**************************************************************/
/* loop over all sediment sizes ends here *********************/
/**************************************************************/



/***************************************************************/
/* bed form computations from van Rijn's formula ***************/
/***************************************************************/

// variables: 
// depth: water depth
// tshear50: value of (shear-critical shear / critical shear), averaged over all sizes
// tshear50_r: similar to tshear50, but the shear stress has been reduced due to grain/dune roughness partitioning
// lnumber: number of sediment sizes
// sedimenthickness: thickness of sediment layer at the bed
// timestep: time step
// bedconc: bed concentration
// bedcellheight: height of bed cell
// velocity: horizontal velocity
// oldbedformheight: old bed form height
// conversioncoefficient: coefficient to transform between suspended
//    and deposited sediment 
// i: index for grid cell in the streamwise direction. Added 11/2-05
// j: index for grid cell in the lateral direction. Added 11/2-05
// CellID: cell number for unstructured grid. zero in a structured grid. Added 11/2-05
// GlobalIter: global iteration number. Added 11/2-05	
// d50: d50 of the bed sediment grain size distribution
// d_mean: mean diameter of the bed grain size distribution
// kinen: turbulent kinetic energy

double computeBedForms(double depth, double d50, double tshear50, int lnumber,
					   double sedimentthickness, double timestep, double bedconc, 
					   double bedcellheight, double velocity, double oldbedformheight,
					   double convercioncoefficient, int i, int j, int CellID, long int GlobalIter,
					   double d_mean, double kinen, double tshear50_r) {
	double a;

	a = tshear50 > 25.0 ? 
		0.0 : depth * 0.11 * pow(d_mean/depth,0.3)*(1-exp(-0.5 * tshear50)) * (25.0 - tshear50);
	a = (double)min(a, 0.5 * sedimentthickness);
	a = (double)min(a, 3.0*depth);
	return a;

}







/**************************************************************************/
/*** computation of bed roughness *****************************************/
/*** only called if FlagRoughness (F 90) is greater or equal to 2 *********/
/**************************************************************************/

// variables: 
// sedimenthickness: thickness of sediment layer at the bed
// FlagRoughness: integer for choice of method to compute the bed roughness
//                given on the F 90 data set
// d90: sieve diameter where 90 % is finer
// XksWall: default roughness, given on the F 16 data set, or computed 
// from the Manning-Stricklers coefficient given on the W 1 data set
// Flag2BedRoughness: Parameter given on the F 103 data set. 
//                    can be used to choose between different options
// bedformheight: bed form height, computed from the previous function
// depth: water depth
// d50: sieve diameter where 50 % is finer, Added 11/2-05
// i: index for grid cell in the streamwise direction. Added 11/2-05
// j: index for grid cell in the lateral direction. Added 11/2-05
// CellID: cell number for unstructured grid. zero in a structured grid. Added 11/2-05
// GlobalIter: global iteration number. Added 11/2-05	
// d_mean: mean diameter of the bed grain size distribution
 
double computeBedRoughness(double sedimentthickness, int FlagRoughness, double d90,
					   double XksWall, int Flag2BedRoughness, double bedformheight,
					   double depth, double d50, int i, int j, int CellID, long int GlobalIter,
					   double d_mean) {

	if(FlagRoughness == 2 && Flag2BedRoughness == 0) {
		if(sedimentthickness > d90)	return 3.0 * d90;
		else return (double)max(XksWall,3.0*sedimentthickness);
		}
	else if((FlagRoughness >= 3 && FlagRoughness <= 5) && Flag2BedRoughness == 0) {
		if(sedimentthickness > d90) return (3.0 * d90 
			+ 1.1 * bedformheight * (1.0 - exp(-25.0*bedformheight/(7.3*depth))));
		else return (double)max(XksWall,3.0*sedimentthickness);
		}
	else return XksWall;

}





